# coding: utf-8
import os
import re
import datetime

import autocomplete_light

from django.contrib import admin
from django import forms
from django.utils.translation import ugettext as _
from django.forms.util import ErrorList
from django.forms.models import BaseInlineFormSet
from django.conf import settings

from jmb.fax.models import Fax, FaxNumber, Cover, Device, Document

class FaxFileInlineFormSet(BaseInlineFormSet):

    def clean(self):

        for form in self.forms:
            cleaned_data = form.cleaned_data
            # verifico sia validita sia non vuoto
            if form.is_valid() and len(cleaned_data.keys()):
                # father object
                fax = cleaned_data['fax']

                content_type = cleaned_data['file'].content_type
                if hasattr(fax, 'device') and fax.device.kpn_url:
                    # verifico i file utilizzabili
                    if content_type not in settings.FAX_ACCEPTED_CONTENT_TYPES[fax.device.type]:
                        # mozilla passa un content_type application/unknown...
                        ext = os.path.splitext(cleaned_data['file'].name)[1].strip('.')
                        if not ext in settings.FAX_ACCEPTED_EXTENSIONS[fax.device.type]:
                            msg = "Not allowed filetype/extension: %s - %s"
                            raise forms.ValidationError(msg % (content_type, ext))
        return self.cleaned_data

class DocumentInline(admin.TabularInline):
    model = Document
    fields = ('file', 'ordering',)
    extra = 0
    formset = FaxFileInlineFormSet
    
class CoverForm(forms.ModelForm):
    generate_cover = forms.BooleanField(required=False)
    class Meta:
        model = Cover

    def clean(self):
        data = super(CoverForm, self).clean()
        if data['generate_cover']:
            return data
        else:
            return {}

class CoverInline(admin.StackedInline):
    model = Cover
    form = CoverForm
    max_num = 1
    fieldsets = (
        (None, {'fields': (
           ('generate_cover',),
            ('to_company', ),
            ('to_person', 'from_person'),
            'pages', 'subject', 'notes'
        )}),
    )

class PhoneChoiceFields(forms.MultipleChoiceField):
    def validate(self, value):
        # I need no validation as the choice may not be present in the db
        return True

class FaxForm(forms.ModelForm):
    recipient_numbers = PhoneChoiceFields(required=True, 
    # FIXME: Funziona correttamente ma con 2 widgets differenti                        
    # spostato nell'init perché questo widget non mostra i numeri quando si visita un 
    # fax esistente
#        widget = autocomplete_light.MultipleChoiceWidget('RecipientAutocomplete', )
    )

    class Meta:
        model = Fax
    inlines = (DocumentInline, CoverInline)

    def __init__(self, *args, **kwargs):
        super(FaxForm, self).__init__(*args, **kwargs)
        ## usa come default l'unico numero disponibile, se ce n'è 1 solo...
        num_list = Device.objects.all()
        initial = None
        if len(num_list) == 1:
            initial = num_list[0].id
        self.fields['device'] = forms.ModelChoiceField(
            queryset=num_list, initial=initial, required=True, label=_("N. Fax"))
        if self.instance.recipient_number:
            self.fields['recipient_numbers'].choices = ([self.instance.recipient_number] * 2,)
        else:
            self.fields['recipient_numbers'].widget = autocomplete_light.MultipleChoiceWidget('RecipientAutocomplete', )

    def clean_recipient_numbers(self):
        # clean the value and store the organization name to be used in recipient
        value_list = self.cleaned_data['recipient_numbers']

        values = []
        self._company_names = {}
        for val in value_list:

            if '[' in val:
                PATTERN = '(?P<company_name>[^()]*?) \[(?P<fax_number>[^()]+)\]'
                m = re.search(PATTERN, val)
                val = m.group('fax_number') 
                self._company_names[val] = m.group('company_name')
            else:
                self._company_names[val] = ''

            values += [val]
        return values


